#version 120
// sample BPlot2D shader

// shader parameters

uniform vec2 screen; // screen pixel sizes
uniform vec4 world;  // box rectangle: (x0, y0, x1-x0, y1-y0)


// indicator function

bool inRect(vec2 p, vec4 lrbt)
{
  return lrbt.x <= p.x && p.x <= lrbt.y 
      && lrbt.z <= p.y && p.y <= lrbt.w;
}


bool inCircle(vec2 p, vec2 c, float r)
{
  return length(p - c) <= r;
}


const vec4 rect1 = vec4(2.0,   4.0, -5.0, 5.0),
           rect2 = vec4(-4.0, -2.0, -5.0, 5.0);

bool indicator(vec2 p)
{
  return (inRect(p, rect1)
       || inRect(p, rect2)
       || inCircle(p, vec2(-2.0, 0.0), 5.0)) && p.x >= -4.0;
}



// auxiliary functions

// maps pixel screen coordinates pc to "world" coordinates
vec2 screen_to_world(vec2 pc)
{
  return world.xy + pc / screen * world.zw;
}


// colors to paint
const vec3 fill_color = vec3(0.1, 0.5, 0.4);
		   
// supersampling
const int offsets = 13;
vec2 off[offsets] = vec2[]
(
  vec2(0.50, 0.50),
  vec2(0.40, 0.65),
  vec2(0.65, 0.60),
  vec2(0.60, 0.35),
  vec2(0.35, 0.40),
  vec2(0.15, 0.80),
  vec2(0.80, 0.85),
  vec2(0.85, 0.20),
  vec2(0.20, 0.15),
  vec2(0.10, 0.45),
  vec2(0.45, 0.90),
  vec2(0.90, 0.55),
  vec2(0.55, 0.10)
);
		   
void main(void)
{
  float hits = 0.0;
  for (int i = 0; i < offsets; ++i)
    hits += (indicator(screen_to_world(gl_FragCoord.xy + off[i]))? 1.0: 0.0);

  gl_FragColor = vec4(fill_color, hits / float(offsets));
}
