#version 120
// sample BPlot2D shader -- simple y(x) graph via indicator: y == y(x)

// shader parameters

uniform vec2 screen; // screen pixel sizes
uniform vec4 world;  // box rectangle: (x0, y0, x1-x0, y1-y0)


// graph

float y(float x)
{
  return sin(x * 3.14) * cos(x) * cos(log2(abs(x) + 1.0)) * sqrt(abs(x) + 1.0);
}


// indicator function

const float tolerance = 0.001;
bool indicator(vec2 p)
{
  return abs(p.y - y(p.x)) <= tolerance * max(world.z, world.w);
}



// auxiliary functions

// maps pixel screen coordinates pc to "world" coordinates
vec2 screen_to_world(vec2 pc)
{
  return world.xy + pc / screen * world.zw;
}


// colors to paint
const vec3 fill_color = vec3(1.0, 0.7, 0.1);
		   
// supersampling
const int offsets = 13;
vec2 off[offsets] = vec2[]
(
  vec2(0.50, 0.50),
  vec2(0.40, 0.65),
  vec2(0.65, 0.60),
  vec2(0.60, 0.35),
  vec2(0.35, 0.40),
  vec2(0.15, 0.80),
  vec2(0.80, 0.85),
  vec2(0.85, 0.20),
  vec2(0.20, 0.15),
  vec2(0.10, 0.45),
  vec2(0.45, 0.90),
  vec2(0.90, 0.55),
  vec2(0.55, 0.10)
);
		   
void main(void)
{
  float hits = 0.0;
  for (int i = 0; i < offsets; ++i)
    hits += (indicator(screen_to_world(gl_FragCoord.xy + off[i]))? 1.0: 0.0);

  gl_FragColor = vec4(fill_color, hits / float(offsets));
}
