#version 110

// GLSL fragment shader drawing classic Mandelbrot set.
// see http://en.wikipedia.org/wiki/Mandelbrot_set
// Author: Dmitri Kuvshinov.

// shader parameters

uniform sampler1D palette;
uniform float t;
uniform float pixel_units_coef;
uniform float width, height;

const int iterations = 255;
const float tc_delta = 1.0 / float(iterations - 1);
const float tc_cutoff = 10.0 * tc_delta;
const vec2 origin_offset = vec2(0.74, 0.48); // 0.75, 0.5

// complex squaring
vec2 cmplx_sqr(vec2 z)
{
  return vec2(z.x * z.x - z.y * z.y, 2.0 * z.x * z.y);
}

// shader procedure

void main(void)
{
  // we do not need either width & height or pixel_units_coef here
  // but the program will not successfully link if all parameters are not being used
  float tt = (0.001 * t) * (t * t);
  vec2 c = (gl_FragCoord.xy / vec2(width, height) - origin_offset) / (0.05 * tt * tt) - vec2(1.25, 0.012), z = c;
  float tc = 0.0;
  for (int i = 0; i < iterations && dot(z, z) <= 4.0; ++i)
  {
    tc += tc_delta;
	z = c + cmplx_sqr(z);
  }
  
  gl_FragColor = tc < tc_cutoff? vec4(0.0):
                 tc < 1.0? texture1D(palette, tc):
                 vec4(pixel_units_coef);
}
