/*
 * This file is part of AV Orchid.
 * Copyright (c) 2014, Dmitri R. Kuvshinov <evetro.here@gmail.com>
 * 
 * Permission to use, copy, modify, and/or distribute this software for
 * any purpose with or without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all copies.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
/**
 * Uniform in [0, 1) random number generator class based upon C++11 <random>.
 */
#pragma once
#include <random>

namespace avo
{

/// Uniform random number generator class (functor) based upon C++11 <random>.
class Uniform_prng
{
public:
  /// number type for values being generated
  using result_type = double;
  /// number type for seeds
  using seed_type = std::random_device::result_type;

  /// full constructor: seed and range
  explicit UniformPrng(seed_type seed, result_type a = 0.0, result_type b = 1.0)
    : rng(seed), dist(a, b) {}

  /// random seed constructor, range may be selected still
  explicit UniformPrng(result_type a = 0.0, result_type b = 1.0)
    : UniformPrng(std::random_device{}(), a, b) {}

  /// get next generated pseudo-random number
  result_type operator()()
  {
    return dist(rng);
  }

private:
  std::mt19937 rng;
  std::uniform_real_distribution<result_type> dist;
};

}
