﻿/*
 * This file is part of AV Orchid.
 * Copyright (c) 2013, Dmitri R. Kuvshinov <evetro.here@gmail.com>
 * 
 * Permission to use, copy, modify, and/or distribute this software for
 * any purpose with or without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all copies.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
/**
 * @file GameScreen.hpp
 *
 * Экран 2. Собственно игра.
 */
#pragma once
#include <functional> // std::function

#include "Screen.hpp"
#include "ViewModelBase.hpp"


/// экран игры
class Game_screen : public AScreen
{
public:

  Game_screen();
  ~Game_screen() override;

  // копирование объектов экрана запрещено
  Game_screen(const Game_screen&) = delete;
  Game_screen& operator=(const Game_screen&) = delete;

  /// внешний обработчик события "игра окончена"
  using Game_over_handler = std::function<void(Score)>;
  /// привязать обработчик события "выход"
  void set_game_over_handler(Game_over_handler handler)
  {
    do_game_over = handler;
  }

  /// проверить, поставлена ли игра на паузу
  bool is_paused() const;

protected:

  void on_select() override;
  void on_idle() override;
  void on_display() override;
  void on_char(unsigned char code, int mouse_x, int mouse_y) override;
  void on_special_key(int code, int mouse_x, int mouse_y) override;
  void on_mouse(int button, int mouse_x, int mouse_y) override;

private:
  class Game *impl; // pimpl
  Game_over_handler do_game_over;
};
